@@ -34,7 +34,8 @@ class SearchEntry(Gtk.SearchEntry):
     """Widget used to filter server list based on user input."""
     def __init__(self):
         super().__init__()
-        self.set_placeholder_text("Press Ctrl+F to search")
+        self.set_name("search-entry")
+        self.props.placeholder_text = "Press Ctrl+F to search"  # pylint: disable=no-member
         self.connect("request-focus", lambda _: self.grab_focus())  # pylint: disable=no-member, disable=line-too-long # noqa: E501 # nosemgrep: python.lang.correctness.return-in-init.return-in-init
 
     @GObject.Signal(name="request_focus", flags=GObject.SignalFlags.ACTION)
